@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=utf-8
 # License: GPL v3 Copyright: 2022, Kovid Goyal <kovid at kovidgoyal.net>
 
 
@@ -10,20 +9,39 @@ import traceback
 from contextlib import suppress
 from functools import partial
 from itertools import count
-from qt.core import (
-    QAbstractItemModel, QAbstractItemView, QCheckBox, QDialog, QDialogButtonBox,
-    QFont, QHBoxLayout, QIcon, QLabel, QMenu, QModelIndex, QPixmap, QPushButton,
-    QRect, QSize, QSplitter, QStackedWidget, Qt, QTreeView, QVBoxLayout, QWidget,
-    pyqtSignal
-)
 from threading import Event, Thread
 
+from qt.core import (
+    QAbstractItemModel,
+    QAbstractItemView,
+    QAction,
+    QCheckBox,
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QHBoxLayout,
+    QIcon,
+    QKeySequence,
+    QLabel,
+    QMenu,
+    QModelIndex,
+    QPixmap,
+    QPushButton,
+    QRect,
+    QSize,
+    QSplitter,
+    QStackedWidget,
+    Qt,
+    QTreeView,
+    QVBoxLayout,
+    QWidget,
+    pyqtSignal,
+)
+
 from calibre import fit_image, prepare_string_for_xml
 from calibre.db import FTSQueryError
 from calibre.ebooks.metadata import authors_to_string, fmt_sidx
-from calibre.gui2 import (
-    config, error_dialog, gprefs, info_dialog, question_dialog, safe_open_url
-)
+from calibre.gui2 import config, error_dialog, gprefs, info_dialog, question_dialog, safe_open_url
 from calibre.gui2.fts.utils import get_db
 from calibre.gui2.library.models import render_pin
 from calibre.gui2.progress_indicator import ProgressIndicator
@@ -31,6 +49,7 @@ from calibre.gui2.ui import get_gui
 from calibre.gui2.viewer.widgets import ResultsDelegate, SearchBox
 from calibre.gui2.widgets import BusyCursor
 from calibre.gui2.widgets2 import HTMLDisplay
+from calibre.utils.localization import ngettext
 
 ROOT = QModelIndex()
 sanitize_text_pat = re.compile(r'\s+')
@@ -43,10 +62,46 @@ def mark_books(*book_ids):
         gui.iactions['Mark Books'].add_ids(book_ids)
 
 
-def jump_to_book(book_id):
+def jump_to_book(book_id, parent=None):
     gui = get_gui()
     if gui is not None:
-        gui.library_view.select_rows((book_id,))
+        parent = parent or gui
+        if gui.library_view.select_rows((book_id,)):
+            gui.raise_and_focus()
+        else:
+            if gprefs['fts_library_restrict_books']:
+                error_dialog(parent, _('Not found'), _('This book was not found in the calibre library'), show=True)
+            else:
+                error_dialog(parent, _('Not found'), _(
+                    'This book is not currently visible in the calibre library.'
+                    ' If you have a search or Virtual library active, try clearing that.'
+                    ' Or click the "Restrict searched books" checkbox in this window to'
+                    ' only search currently visible books.'), show=True)
+
+
+def show_in_viewer(book_id, text, fmt):
+    text = text.strip('…').replace('\x1d', '').replace('\xa0', ' ')
+    text = sanitize_text_pat.sub(' ', text)
+    gui = get_gui()
+    if gui is not None:
+        if fmt in config['internally_viewed_formats']:
+            gui.iactions['View'].view_format_by_id(book_id, fmt, open_at=f'search:{text}')
+        else:
+            gui.iactions['View'].view_format_by_id(book_id, fmt)
+
+
+def open_book(results, match_index=None):
+    gui = get_gui()
+    if gui is None:
+        return
+    book_id = results.book_id
+    if match_index is None:
+        gui.iactions['View'].view_historical(book_id)
+        return
+    result_dict = results.result_dicts[match_index]
+    formats = results.formats[match_index]
+    from calibre.gui2.actions.view import preferred_format
+    show_in_viewer(book_id, result_dict['text'], preferred_format(formats))
 
 
 class SearchDelegate(ResultsDelegate):
@@ -419,13 +474,29 @@ class ResultsView(QTreeView):
         results, match = self.m.data_for_index(index)
         m = QMenu(self)
         if results:
-            m.addAction(QIcon.ic('lt.png'), _('Jump to this book in the library'), partial(jump_to_book, results.book_id))
+            m.addAction(QIcon.ic('lt.png'), _('Jump to this book in the library'), partial(jump_to_book, results.book_id, self))
             m.addAction(QIcon.ic('marked.png'), _('Mark this book in the library'), partial(mark_books, results.book_id))
+            if match is not None:
+                match = index.row()
+                m.addAction(QIcon.ic('view.png'), _('View this book at this search result'), partial(open_book, results, match_index=match))
+            else:
+                m.addAction(QIcon.ic('view.png'), _('View this book'), partial(open_book, results))
         m.addSeparator()
         m.addAction(QIcon.ic('plus.png'), _('Expand all'), self.expandAll)
         m.addAction(QIcon.ic('minus.png'), _('Collapse all'), self.collapseAll)
         m.exec(self.mapToGlobal(pos))
 
+    def view_current_result(self):
+        idx = self.currentIndex()
+        if idx.isValid():
+            results, match = self.m.data_for_index(idx)
+            if results:
+                if match is not None:
+                    match = idx.row()
+                open_book(results, match)
+                return True
+        return False
+
 
 class Spinner(ProgressIndicator):
 
@@ -491,7 +562,8 @@ class SearchInputPanel(QWidget):
         self.related = rw = QCheckBox(_('&Match on related words'))
         rw.setToolTip('<p>' + _(
             'With this option searching for words will also match on any related words (supported in several languages). For'
-            ' example, in the English language: <i>correction</i> matches <i>correcting</i> and <i>corrected</i> as well'))
+            ' example, in the English language: {0} matches {1} and {2} as well').format(
+            '<i>correction</i>', '<i>correcting</i>', '<i>corrected</i>'))
         rw.setChecked(gprefs['fts_library_use_stemmer'])
         rw.stateChanged.connect(lambda state: gprefs.set('fts_library_use_stemmer', state != Qt.CheckState.Unchecked.value))
         self.summary = s = QLabel(self)
@@ -510,6 +582,9 @@ class SearchInputPanel(QWidget):
     def clear_history(self):
         self.search_box.clear_history()
 
+    def set_search_text(self, text):
+        self.search_box.setText(text)
+
     def start(self):
         self.pi.start()
 
@@ -534,6 +609,9 @@ class ResultDetails(QWidget):
 
     def __init__(self, parent=None):
         super().__init__(parent)
+        self.jump_action = ac = QAction(self)
+        ac.triggered.connect(self.jump_to_current_book)
+        ac.setShortcut(QKeySequence('Ctrl+S', QKeySequence.SequenceFormat.PortableText))
         self.key = None
         self.pixmap_label = pl = QLabel(self)
         pl.setScaledContents(True)
@@ -571,6 +649,10 @@ class ResultDetails(QWidget):
                     ' to see updated results.'), show=True)
                 self.remove_book_from_results.emit(self.current_book_id)
 
+    def jump_to_current_book(self):
+        if self.current_book_id > -1:
+            jump_to_book(self.current_book_id)
+
     def results_anchor_clicked(self, url):
         if self.current_book_id > 0 and url.scheme() == 'book':
             book_id, result_num, fmt = url.path().strip('/').split('/')
@@ -631,7 +713,8 @@ class ResultDetails(QWidget):
             text += '<p>' + _('{series_index} of {series}').format(series_index=sidx, series=series) + '</p>'
         ict = '<img valign="bottom" src="calibre-icon:///{}" width=16 height=16>'
         text += '<p><a href="calibre://jump" title="{1}">{2}\xa0{0}</a>\xa0\xa0\xa0 '.format(
-            _('Select'), '<p>' + _('Scroll to this book in the calibre library book list and select it.'), ict.format('lt.png'))
+            _('Select'), '<p>' + _('Scroll to this book in the calibre library book list and select it [{}]').format(
+                self.jump_action.shortcut().toString(QKeySequence.SequenceFormat.NativeText)), ict.format('lt.png'))
         text += '<a href="calibre://mark" title="{1}">{2}\xa0{0}</a></p>'.format(
             _('Mark'), '<p>' + _(
                 'Put a pin on this book in the calibre library, for future reference.'
@@ -644,12 +727,13 @@ class ResultDetails(QWidget):
             text += '<p><a href="calibre://unindex" title="{1}">{2}\xa0{0}</a>'.format(
                 _('Un-index'), _('This book has been deleted from the library but is still present in the'
                                           ' full text search index. Remove it.'), ict.format('trash.png'))
+        text += '<p>' + _('This book may have more than one match, only a single match per book is shown.')
         self.book_info.setHtml(text)
 
     def render_results(self, results, individual_match=None):
         html = []
         space_pat = re.compile(r'\s+')
-        markup_pat = re.compile('\x1d')
+        markup_pat = re.compile(r'\x1d')
 
         def markup_text(text):
             count = 0
@@ -719,7 +803,7 @@ p { margin: 0; }
 <div class="bq">NEAR("people" "in Asia" "try")<br>NEAR("Kovid" "calibre", 30)</div>
 <p>Here, 30 is the most words allowed between near groups. Defaults to 10 when unspecified.</p>
 
-<div style="margin-top: 1em"><a href="{fts_url}">Full syntax reference</a></div>
+<div style="margin-top: 1em"><a href="{fts_url}">Complete syntax reference</a></div>
 ''' + '</div>').format(fts_url=fts_url))
         hp.setFocusPolicy(Qt.FocusPolicy.NoFocus)
         hp.document().setDocumentMargin(0)
@@ -801,22 +885,25 @@ class ResultsPanel(QWidget):
         if st is not None:
             s.restoreState(st)
 
+    @property
+    def jump_to_current_book_action(self):
+        return self.details.result_details.jump_action
+
+    def view_current_result(self):
+        return self.results_view.view_current_result()
+
     def clear_history(self):
         self.sip.clear_history()
 
+    def set_search_text(self, text):
+        self.sip.set_search_text(text)
+
     def remove_book_from_results(self, book_id):
         self.results_view.m.remove_book(book_id)
 
     def show_in_viewer(self, book_id, result_num, fmt):
         r = self.results_view.m.get_result(book_id, result_num)
-        text = r['text'].strip('…').replace('\x1d', '').replace('\xa0', ' ')
-        text = sanitize_text_pat.sub(' ', text)
-        gui = get_gui()
-        if gui is not None:
-            if fmt in config['internally_viewed_formats']:
-                gui.iactions['View'].view_format_by_id(book_id, fmt, open_at=f'search:{text}')
-            else:
-                gui.iactions['View'].view_format_by_id(book_id, fmt)
+        show_in_viewer(book_id, r['text'], fmt)
 
     def request_stop_search(self):
         if question_dialog(self, _('Are you sure?'), _('Abort the current search?')):
@@ -866,7 +953,7 @@ if __name__ == '__main__':
     from calibre.library import db
     app = Application([])
     d = QDialog()
-    d.sizeHint = lambda : QSize(1000, 680)
+    d.sizeHint = lambda: QSize(1000, 680)
     l = QVBoxLayout(d)
     bb = QDialogButtonBox(d)
     bb.accepted.connect(d.accept), bb.rejected.connect(d.reject)
